const Validator = require('./validator');

/**
 * Create an instance of HandlerValidator
 *
 * @param {Function} fn function that you will handle with args validation
 * @param {Object|Array} schema Enumerated object with schemas | Array with Schemas
 * @return {Function} A Handled Function
 */

module.exports = class HandlerValidator extends Validator {
  constructor(fn, schema) {
    if (typeof fn !== 'function')
      throw new TypeError(`The argument in position '0' must be a function, received type of ${typeof fn}`);

    const validator = super(schema);

    function apply(fn, thisArg, argArr) {
      const argObj = { ...argArr };

      validator(argObj);

      const parsedArg = new Array(fn.length);

      Object.entries(argObj).forEach(([index, value]) => {
        parsedArg[index] = value;
      });

      return fn(...parsedArg);
    };

    return new Proxy(fn, { apply });
  }
};